#! python2
# -*- coding: utf-8 -*-

from naoqi import ALProxy
import qi
import time
import json
import paho.mqtt.client as mqtt
# from termcolor import colored # termcolor is not available in Python 2.7

# Must use "".format() instead of f-strings to be compatible with Python 2.7
# NAOqi requires Python 2.7

# MQTT settings
broker_addres = "test.mosquitto.org"
broker_port = 1883
client_id = "NAO_Client"

topic_speak = "nao/test/input/speak"
topic_func = "nao/test/input/func"
topic_log = "nao/test/log"

# NAO settings
NAO_IP = "192.168.0.185"
NAO_PORT = 9559

# Create qi session
session = qi.Session()
tcp_link = u"tcp://{}:{}".format(NAO_IP, NAO_PORT)
session.connect(tcp_link)

# Create NAOqi proxys
tts = session.service("ALTextToSpeech")
ams = session.service("ALAnimatedSpeech")
# Naoqi language settings
tts.setLanguage("Spanish")
# insert tts settings here

# MQTT callbacks
def on_connect(client, userdata, flags, rc):
    """ Connection callback:
        - Publishes a connection log message
        - Subscribes to the topics "speak" and "func"
    """
    if rc == 0:
        print("Connected successfully to MQTT broker")
        conn_msg = "Client '{}' connected to MQTT broker".format(client_id)
        client.publish(topic_log, conn_msg)
        client.subscribe(topic_speak)
        client.subscribe(topic_func)
    else:
        print("Falied to connect to MQTT broker")

def disconnect_with_msg(client):
    """ Disconnect from the MQTT broker and publishes a disconnection log message """
    print("Disconnecting from the MQTT broker...")
    disconn_msg = "Client '{}' disconnected from MQTT broker".format(client_id)
    client.publish(topic_log, disconn_msg)
    client.disconnect()
    client.loop_stop()

def on_message(client, userdata, message):
    """ Message callback:
        - Prints the received message to the terminal specifying the topic color
    """ 
    # Get the topic and the message data (decoded from JSON format)
    topic = message.topic
    data = json.loads(message.payload)

    # Print the message to the terminal - non-colored
    role = data['role']
    content = data['content']
    
    print(u"\nFrom Topic: {}".format(topic))
    print(u"{}: {}".format(role, content))

    #tts.say(content.encode('utf-8')) # NAOqi requires utf-8 encoding
    ams.say(content.encode('utf-8')) # NAOqi requires utf-8 encoding




def main():
    
    # Create client instance and set callbacks
    client = mqtt.Client(client_id)
    client.on_connect = on_connect
    client.on_message = on_message

    # Connect to the broker
    raw_input("\nPress Enter to start the NAO client...")
    client.connect(broker_addres, broker_port)
    client.loop_start()
    time.sleep(3)

    try:
        while True:
            time.sleep(1) # Infinite loop to keep the client running

    except KeyboardInterrupt:
        # Ctrl+C interrupt, disconnect from the broker and exit
        disconnect_with_msg(client)
        session.close()
        print("Session closed...")
        print("\nProgram terminated by user...")

if __name__ == "__main__":
    main()