#! python2
# -*- coding: utf-8 -*-

# Activate the SpeechRecognition module with:
# cd pepperspeechrecognition
# python module_speechrecognition.py --pip 192.168.0.185


import naoqi
import os
import sys
import time
import qi
import json
import subprocess
import paho.mqtt.client as mqtt

class NaoListener:

    # MQTT Settings
    broker_addres = "test.mosquitto.org"
    broker_port = 1883
    client_id = "STT_Client"

    topic_speech = "nao/test/speech"
    # topic_speak = "nao/test/input/speak"
    # topic_func = "nao/test/input/func"
    topic_log = "nao/test/log"

    # NAO Settings
    NAO_IP = "192.168.0.185"
    NAO_PORT = 9559

    def __init__(self, session):
        self.session = session
        self.mqtt_client = mqtt.Client(self.client_id)

        self.session.connect("tcp://" + self.NAO_IP + ":" + str(self.NAO_PORT))
        self.mem = self.session.service("ALMemory")
        self.SpeechRecognition = self.session.service("SpeechRecognition")
        
        # Subscribe to the SpeechRecognition event and set the callback
        self.event = self.mem.subscriber("SpeechRecognition")
        self.event.signal.connect(self.on_speech_recognition)

        # Subscribe to the speechdone event and set the callback
        self.ttsend = self.mem.subscriber("ALTextToSpeech/TextDone")
        self.amsend = self.mem.subscriber("ALAnimatedSpeech/EndOfAnimatedSpeech")

        self.ttsend.signal.connect(self.on_speech_done)
        self.amsend.signal.connect(self.on_speech_done)


        # Set the callbacks for the MQTT client
        self.mqtt_client.on_connect = self.on_connect

    def calibrate(self):
        self.SpeechRecognition.start()
        self.SpeechRecognition.setHoldTime(2.5)
        self.SpeechRecognition.setIdleReleaseTime(1.0)
        self.SpeechRecognition.setMaxRecordingDuration(10)
        self.SpeechRecognition.setLookaheadDuration(0.5)
        self.SpeechRecognition.setLanguage("es")
        self.SpeechRecognition.calibrate()
        self.SpeechRecognition.setAutoDetectionThreshold(5)
    



    def on_speech_recognition(self, value):
        print("\nSpeech recognized:")
        # print(type(value))
        print(value.decode('utf-8'))

        # Parse into a JSON string (object)
        speech = json.dumps({"text": value.rstrip()})
        # Publish the recognized speech to the speech topic
        self.mqtt_client.publish(self.topic_speech, speech)
        self.SpeechRecognition.disableAutoDetection()
    
    def on_speech_done(self, value):
        if isinstance(value, bool) and value == True:
            self.SpeechRecognition.enableAutoDetection()


    def on_connect(self, client, userdata, flags, rc):
        """ Connection callback, publishes a connection log message """
        if rc == 0:
            print("Connected successfully to MQTT broker")
            conn_msg = "Client '{}' connected to MQTT broker".format(self.client_id)
            client.publish(self.topic_log, conn_msg)
        else:
            print("Falied to connect to MQTT broker")

    def disconnect_with_msg(self):
        """ Disconnect from the MQTT broker and publishes a disconnection log message """
        print("Disconnecting from the MQTT broker...")
        disconn_msg = "Client '{}' disconnected from MQTT broker".format(self.client_id)
        self.mqtt_client.publish(self.topic_log, disconn_msg)
        self.mqtt_client.disconnect()
        self.mqtt_client.loop_stop()

    def start(self):
        # Connect to the MQTT broker
        print("Connecting to the MQTT broker...")
        self.mqtt_client.connect(self.broker_addres, self.broker_port)
        # Start the MQTT client loop
        self.mqtt_client.loop_start()

        self.SpeechRecognition.enableAutoDetection()

        try:
            while True:
                time.sleep(1)
        except KeyboardInterrupt:
            print("Interrupted by user, shutting down...")
            self.disconnect_with_msg()
            self.session.close()
            print("Session closed")
            print("\nProgram terminated by user...")
            sys.exit(0)

def run_sr_module(folder_path, script_name, options):
    command = 'cd /D {0} && python {1} {2}'.format(folder_path, script_name, options)
    # print("\nRunning command: " + command)
    subprocess.Popen(['cmd', '/C', 'start', 'cmd', '/K', command])

def main():
    nao_ip = "192.168.0.185"
    
    # Run the SpeechRecognition module
    current_path = os.path.dirname(os.path.realpath(__file__))
    script_name = "module_speechrecognition.py"
    options = "--pip " + nao_ip
    folder_path = os.path.join(current_path, "pepperspeechrecognition")
    process = run_sr_module(folder_path, script_name, options)
    
    # Wait for the SpeechRecognition module to start
    print("\nWaiting for the SpeechRecognition module to start...")
    time.sleep(5)

    # Create a qi session
    session = qi.Session()
    # Instantiate the NaoListener object and calibrate the SpeechRecognition module
    listener = NaoListener(session)
    listener.calibrate()
    print("\nCalibrating the SpeechRecognition module...")
    time.sleep(2)

    # Start the listener loop
    raw_input("\nPress Enter to start the listener...")
    listener.start()

if __name__ == "__main__":
    main()


