#! python3
# -*- coding: utf-8 -*-

# H:\miniconda3\Scripts\activate tfg1
# cd E:\code\fcalling\fun1

"""
This script initializes and starts the chatbot server.

Usage:
- Run this script to start the chatbot.

Requirements:
- Python 3.x
- gpt_chatbot module
- gpt_functions module

Global Variables:
- STREAM_MODE_ON: A boolean indicating whether stream mode is enabled or not. (Not implemented)
- TOKEN_USAGE: A boolean indicating whether token usage is enabled or not.
- GPT_MODEL: The name of the GPT model to be used.
- PROJECT_PATH: The path to the project directory.

System Prompt:
- SYSTEM_PROMPT: A string containing the system prompt for the chatbot. It provides a description of the chatbot's purpose and strict instructions for response length and character usage.

Options:
- options: A dictionary containing various options for the chatbot, including stream mode, token usage, GPT model, system prompt, and project path.

Functions:
- main(): The main function that initializes and starts the chatbot.

"""

from pathlib import Path
from gpt_chatbot import AIChatbot
from gpt_functions import get_empty_conversation
import os

# GLOBAL VARIABLES
STREAM_MODE_ON = False # NOT IMPLEMENTED
TOKEN_USAGE = True
GPT_MODEL = "gpt-3.5-turbo-0125"
PROJECT_PATH = Path(__file__).resolve().parent

# SYSTEM PROMPT
SYSTEM_PROMPT = "Descripción: Eres un asistente personal integrado en un robot humanoide modelo NAO fabricado por la empresa SoftBank Robotics. \
Tu propósito es ejercer de asistente conversacional y de entretenimiento para el usuario. \
Puedes mantener conversaciones, contar chistes, adivinanzas, curiosidades, y mucho más.\n"
SYSTEM_PROMPT+= """INSTRUCCIONES ESTRICTAS:
- PROPORCIONA UNICAMENTE RESPUESTAS DE 25 PALABRAS O MENOS!
- UTILIZA UNICAMENTE CARACTERES DEL CODIGO ASCII
- NO ENVIES EMOJIS, STICKERS, GIFS, O ARCHIVOS ADJUNTOS BAJO NINGUNA CIRCUNSTANCIA!"""

options = {
    'stream_mode': STREAM_MODE_ON,
    'token_usage': TOKEN_USAGE,
    'gpt_model': GPT_MODEL,
    'system_prompt': SYSTEM_PROMPT,
    'project_path': PROJECT_PATH
}


def main():
    """
    Initializes and starts the chatbot.

    Steps:
    1. Load an empty conversation.
    2. Create the chatbot object.
    3. Start the chatbot.

    """
    # Force the terminal to use color codes
    os.system('color')
    # Load an empty conversation
    conversation = get_empty_conversation(SYSTEM_PROMPT)
    # Create the chatbot object
    chatbot = AIChatbot(conversation, options)
    # Start the chatbot
    input("\nPress Enter to start the chatbot...")
    chatbot.start()


if __name__ == "__main__":
    main()